@@ -1,6 +1,7 @@
 """Define new Ops from existing Ops"""
 from collections import OrderedDict
 from functools import partial
+from typing import List, Optional
 
 import aesara.tensor as aet
 from aesara.compile.function.pfunc import rebuild_collect_shared
@@ -11,6 +12,7 @@ from aesara.configdefaults import config
 from aesara.gradient import DisconnectedType, Rop, grad
 from aesara.graph.basic import (
     Apply,
+    Constant,
     Variable,
     clone_replace,
     graph_inputs,
@@ -18,7 +20,7 @@ from aesara.graph.basic import (
 )
 from aesara.graph.fg import FunctionGraph
 from aesara.graph.null_type import NullType
-from aesara.graph.op import Op, ops_with_inner_function
+from aesara.graph.op import HasInnerGraph, Op
 from aesara.graph.opt import in2out, local_optimizer
 from aesara.tensor.basic_opt import ShapeFeature
 
@@ -76,111 +78,16 @@ def infer_shape(outs, inputs, input_shapes):
     return ret
 
 
-class OpFromGraph(Op):
+class OpFromGraph(Op, HasInnerGraph):
     r"""
-    This creates an ``Op`` from inputs and outputs lists of variables.
+    This creates an `Op` from inputs and outputs lists of variables.
     The signature is similar to :func:`aesara.function <aesara.function>`
-    and the resulting ``Op``'s perform will do the same operation as::
+    and the resulting `Op`'s perform will do the same operation as::
 
         orig_function(inputs, outputs, **kwargs)
 
     Currently does not support ``updates`` or ``givens`` argument.
 
-    Parameters
-    ----------
-
-    inputs: list of :class:`Variable <aesara.graph.basic.Variable>`
-
-    outputs: list of :class:`Variable <aesara.graph.basic.Variable>`
-
-    inline: bool, optional
-        Defaults to ``False``
-
-        ``True`` : Cause the Op's original graph being used during
-        compilation, the Op will not be visible in the compiled
-        graph but rather its internal graph.
-
-        ``False`` : will use a pre-compiled function inside.
-
-    grad_overrides : single or list of {'default', OpFromGraph, callable, Variable with special type}, optional
-        Defaults to ``'default'``.
-        This argument is mutually exclusive with lop_overrides.
-
-        ``'default'`` : Do not override, use default grad() result
-
-        OpFromGraph instance : Override with another OpFromGraph, should
-        accept inputs as the same order and types of ``inputs`` and ``output_grads``
-        arguments as one would specify in grad() method.
-
-        callable : Should take two args: ``inputs`` and ``output_grads``.
-        Each argument is expected to be a list of :class:`Variable <aesara.graph.basic.Variable>`.
-        Must return list of :class:`Variable <aesara.graph.basic.Variable>`.
-
-        Variable :
-            ``NullType() instance`` : Treat as non-differentiable
-            ``DisconnectedType() instance`` : Treat as disconnected gradient, numerically gives zero
-
-        list: Each OpFromGraph/callable must return a single
-        :class:`Variable <aesara.graph.basic.Variable>`. Each list element corresponds to gradient of
-        a specific input, length of list must be equal to number of inputs.
-
-    lop_overrides : single or list of {'default', OpFromGraph, callable, Variable with special type}, optional
-        Defaults to ``'default'``.
-        This argument is mutually exclusive with ``grad_overrides``.
-
-        ``'default'`` : Do not override, use default L_op() result
-
-        OpFromGraph instance : Override with another OpFromGraph, should
-        accept inputs as the same order and types of ``inputs``, ``outputs`` and ``output_grads``
-        arguments as one would specify in grad() method.
-
-        callable : Should take three args: ``inputs``, ``outputs`` and ``output_grads``.
-        Each argument is expected to be a list of :class:`Variable <aesara.graph.basic.Variable>`.
-        Must return list of :class:`Variable <aesara.graph.basic.Variable>`.
-
-        Variable :
-            ``NullType() instance`` : Treat as non-differentiable
-            ``DisconnectedType() instance`` : Treat as disconnected gradient, numerically gives zero
-
-        list: Each OpFromGraph/callable must return a single
-        :class:`Variable <aesara.graph.basic.Variable>`. Each list element corresponds to gradient of
-        a specific input, length of list must be equal to number of inputs.
-
-    rop_overrides : single or list of {'default', OpFromGraph, callable, Variable with special type}, optional
-        Defaults to ``default``.
-
-        ``'default'`` : Do not override, use default R_op() result
-
-        OpFromGraph instance : Override with another OpFromGraph, should
-        accept inputs as the same order and types of ``inputs`` and ``eval_points``
-        arguments as one would specify in R_op() method.
-
-        callable : Should take two args: ``inputs`` and ``eval_points``.
-        Each argument is expected to be a list of :class:`Variable <aesara.graph.basic.Variable>`.
-        Must return list of :class:`Variable <aesara.graph.basic.Variable>`.
-
-        Variable :
-            ``NullType() instance`` : Treat as non-differentiable
-            ``DisconnectedType() instance`` : Treat as zero since DisconnectedType is not yet supported in R_op
-
-        list: Each OpFromGraph/callable must return a single
-        :class:`Variable <aesara.graph.basic.Variable>`. Each list element corresponds
-        to a specific output of R_op, length of list must be equal to number of outputs.
-
-    connection_pattern : list of list
-
-        If not ``None``, this will be used as the connection_pattern
-        for this op.
-
-    name : string, optional
-        A name for debugging purposes
-
-    \*\*kwargs : optional
-        Check
-        :func:`orig_function <aesara.compile.function.types.orig_function>`
-        for more arguments, only works when not inline.
-
-
     .. TODO:
         - examples for a multi-layer mlp. where?
         - __hash__, __eq__ otherwise won't merge, try
@@ -321,23 +228,118 @@ class OpFromGraph(Op):
 
     def __init__(
         self,
-        inputs,
-        outputs,
-        inline=False,
-        lop_overrides="default",
-        grad_overrides="default",
-        rop_overrides="default",
-        connection_pattern=None,
-        name=None,
+        inputs: List[Variable],
+        outputs: List[Variable],
+        inline: bool = False,
+        lop_overrides: str = "default",
+        grad_overrides: str = "default",
+        rop_overrides: str = "default",
+        connection_pattern: Optional[List[List[bool]]] = None,
+        name: Optional[str] = None,
         **kwargs,
     ):
-        if not isinstance(outputs, list):
-            raise TypeError(f"outputs must be list, got {type(outputs)}")
+        """
+        Parameters
+        ----------
+        inputs
+            The inputs to the graph.
+        outputs
+            The outputs to the graph.
+        inline
+            Defaults to ``False``
+
+            ``True`` : Cause the :class:`Op`'s original graph being used during
+            compilation, the :class:`Op` will not be visible in the compiled
+            graph but rather its internal graph.
+
+            ``False`` : will use a pre-compiled function inside.
+        grad_overrides
+            Defaults to ``'default'``.
+            This argument is mutually exclusive with ``lop_overrides``.
+
+            ``'default'`` : Do not override, use default grad() result
+
+            `OpFromGraph`: Override with another `OpFromGraph`, should
+            accept inputs as the same order and types of ``inputs`` and ``output_grads``
+            arguments as one would specify in :meth:`Op.grad`() method.
+
+            `callable`: Should take two args: ``inputs`` and ``output_grads``.
+            Each argument is expected to be a list of :class:`Variable `.
+            Must return list of :class:`Variable `.
+        lop_overrides
+            Defaults to ``'default'``.
+
+            This argument is mutually exclusive with ``grad_overrides``.
+
+            These options are similar to the ``grad_overrides`` above, but for
+            the :meth:`Op.L_op` method.
+
+            ``'default'``: Do not override, use the default :meth:`Op.L_op` result
+
+            `OpFromGraph`: Override with another `OpFromGraph`, should
+            accept inputs as the same order and types of ``inputs``,
+            ``outputs`` and ``output_grads`` arguments as one would specify in
+            :meth:`Op.grad` method.
+
+            `callable`: Should take three args: ``inputs``, ``outputs`` and ``output_grads``.
+            Each argument is expected to be a list of :class:`Variable`.
+            Must return list of :class:`Variable`.
+
+            `NullType` instance: Treat as non-differentiable
+            `DisconnectedType` instance: Treat as disconnected gradient,
+            numerically gives zero
+
+            ``list``: Each `OpFromGraph`/callable must return a single
+            :class:`Variable`. Each list element corresponds to gradient of
+            a specific input, length of list must be equal to number of inputs.
+
+        rop_overrides
+            One of ``{'default', OpFromGraph, callable, Variable}``.
+
+            Defaults to ``'default'``.
+
+            ``'default'``: Do not override, use the default :meth:`Op.R_op` result
+
+            `OpFromGraph`: Override with another `OpFromGraph`, should
+            accept inputs as the same order and types of ``inputs`` and ``eval_points``
+            arguments as one would specify in :meth:`Op.R_op` method.
+
+            `callable`: Should take two args: ``inputs`` and ``eval_points``.
+            Each argument is expected to be a list of :class:`Variable`.  Must
+            return list of :class:`Variable`.
+
+            `NullType` instance: Treat as non-differentiable `DisconnectedType`
+            instance: Treat as zero since `DisconnectedType` is not yet supported
+            in :meth:`Op.R_op`.
+
+            ``list``:
+            Each :class:`OpFromGraph`/callable must return a single
+            :class:`Variable <aesara.graph.basic.Variable>`. Each list element
+            corresponds to a specific output of :meth:`Op.R_op`, length of list
+            must be equal to number of outputs.  connection_pattern If not
+            ``None``, this will be used as the connection_pattern for this
+            :class:`Op`.
+        name
+            A name for debugging purposes.
+        kwargs
+            Check :func:`orig_function` for more arguments, only works when not
+            inline.
+        """
+
+        if not (isinstance(inputs, list) and isinstance(outputs, list)):
+            raise TypeError("Inputs and outputs must be lists")
+
         for i in inputs + outputs:
             if not isinstance(i, Variable):
-                raise TypeError("inputs and outputs must be Variable instances", i)
+                raise TypeError(
+                    f"Inputs and outputs must be Variable instances; got {i}"
+                )
+            if i in inputs and isinstance(i, Constant):
+                raise TypeError(f"Constants not allowed as inputs; {i}")
+
         if "updates" in kwargs or "givens" in kwargs:
-            raise TypeError("updates and givens are not allowed here")
+            raise NotImplementedError("Updates and givens are not allowed here")
+
         self.is_inline = inline
         # To correctly support shared variables the inner fct should
         # not see them. Otherwise there is a problem with the gradient.
@@ -363,8 +365,8 @@ class OpFromGraph(Op):
         assert not update_expr
         assert not shared_inputs
 
-        self.local_inputs = local_inputs
-        self.local_outputs = local_outputs
+        self._inner_inputs = local_inputs
+        self._inner_outputs = local_outputs
         self.inputs = inputs
         self.outputs = outputs
         self.kwargs = kwargs
@@ -411,8 +413,8 @@ class OpFromGraph(Op):
         converts self._lop_op from user supplied form to type(self) instance
 
         """
-        local_inputs = self.local_inputs
-        local_outputs = self.local_outputs
+        local_inputs = self.inner_inputs
+        local_outputs = self.inner_outputs
         inp_len = len(local_inputs)
         lop_op = self._lop_op
 
@@ -424,7 +426,7 @@ class OpFromGraph(Op):
             )
             if self._lop_type == "grad":
                 needed_ninps = inp_len + len(local_outputs)
-                ninps = len(lop_op.local_inputs)
+                ninps = len(lop_op.inner_inputs)
                 if needed_ninps != ninps:
                     raise ValueError(self.OV_INP_LEN_ERR_MSG % (needed_ninps, ninps))
                 # make a wrapper callable
@@ -435,7 +437,7 @@ class OpFromGraph(Op):
             elif self._lop_type == "lop":
                 # OfG can be directly used in L_op format
                 needed_ninps = inp_len + 2 * len(local_outputs)
-                ninps = len(lop_op.local_inputs)
+                ninps = len(lop_op.inner_inputs)
                 if needed_ninps != ninps:
                     raise ValueError(self.OV_INP_LEN_ERR_MSG % (needed_ninps, ninps))
                 self._lop_op_is_cached = True
@@ -551,8 +553,8 @@ class OpFromGraph(Op):
         converts self._rop_op from user supplied form to type(self) instance
 
         """
-        local_inputs = self.local_inputs
-        local_outputs = self.local_outputs
+        local_inputs = self.inner_inputs
+        local_outputs = self.inner_outputs
         out_len = len(local_outputs)
         rop_op = self._rop_op
 
@@ -728,7 +730,7 @@ class OpFromGraph(Op):
         return ret_l
 
     def make_node(self, *inputs):
-        num_expected_inps = len(self.local_inputs) - len(self.shared_inputs)
+        num_expected_inps = len(self.inner_inputs) - len(self.shared_inputs)
         if len(inputs) != num_expected_inps:
             raise ValueError(
                 f"Expected {int(num_expected_inps)} inputs, got {len(inputs)}"
@@ -741,8 +743,6 @@ class OpFromGraph(Op):
             list(inputs) + self.shared_inputs,
             [type() for type in self.output_types],
         )
-        apply_node.local_inputs = self.local_inputs
-        apply_node.local_outputs = self.local_outputs
         return apply_node
 
     def connection_pattern(self, node):
@@ -753,13 +753,13 @@ class OpFromGraph(Op):
         if self._connection_pattern is not None:
             return self._connection_pattern
 
-        inp_len = len(self.local_inputs)
-        out_len = len(self.local_outputs)
-        cpmat_self = io_connection_pattern(self.local_inputs, self.local_outputs)
+        inp_len = len(self.inner_inputs)
+        out_len = len(self.inner_outputs)
+        cpmat_self = io_connection_pattern(self.inner_inputs, self.inner_outputs)
 
         lop_op = self.get_lop_op()
         cpmat_grad = io_connection_pattern(
-            lop_op.local_inputs[inp_len:], lop_op.local_outputs
+            lop_op.inner_inputs[inp_len:], lop_op.inner_outputs
         )
 
         # cpmat_self |= cpmat_grad.T
@@ -781,7 +781,7 @@ class OpFromGraph(Op):
     def infer_shape(self, fgraph, node, shapes):
 
         # TODO: Use `fgraph.shape_feature` to do this instead.
-        out_shapes = infer_shape(self.local_outputs, self.local_inputs, shapes)
+        out_shapes = infer_shape(self.inner_outputs, self.inner_inputs, shapes)
 
         # Clone the output shape so that shape are computed from outer inputs.
         # Note:
@@ -791,7 +791,7 @@ class OpFromGraph(Op):
         # each shape call. Aesara optimizer will clean this up later, but this
         # will make extra work for the optimizer.
 
-        repl = dict(zip(self.local_inputs, node.inputs))
+        repl = dict(zip(self.inner_inputs, node.inputs))
         clone_out_shapes = [s for s in out_shapes if isinstance(s, tuple)]
         cloned = clone_replace(sum(clone_out_shapes, ()), replace=repl)
         ret = []
@@ -806,12 +806,24 @@ class OpFromGraph(Op):
 
         return ret
 
-    def prepare_node(self, node, storage_map, compute_map, impl):
-        if not hasattr(self, "fn") and impl == "py":
-            self.fn = orig_function(
-                self.local_inputs, self.local_outputs, **self.kwargs
-            )
-            self.fn.trust_input = True
+    @property
+    def fn(self):
+        """Lazily compile the inner function graph."""
+        if getattr(self, "_fn", None) is not None:
+            return self._fn
+
+        self._fn = orig_function(self.inner_inputs, self.inner_outputs, **self.kwargs)
+        self._fn.trust_input = True
+
+        return self._fn
+
+    @property
+    def inner_inputs(self):
+        return self._inner_inputs
+
+    @property
+    def inner_outputs(self):
+        return self._inner_outputs
 
     def perform(self, node, inputs, outputs):
         variables = self.fn(*inputs)
@@ -833,7 +845,7 @@ def inline_ofg_expansion(fgraph, node):
     if not op.is_inline:
         return False
     return clone_replace(
-        op.local_outputs, {u: v for u, v in zip(node.op.local_inputs, node.inputs)}
+        op.inner_outputs, {u: v for u, v in zip(op.inner_inputs, node.inputs)}
     )
 
 
@@ -846,7 +858,3 @@ optdb.register(
     "fast_compile",
     "fast_run",
 )
-
-# Since OpFromGraph contains an Aesara compiled function,
-# we should let DebugMode know about it
-ops_with_inner_function[OpFromGraph] = "fn"
