@@ -229,18 +229,6 @@ class Type(MetaObject):
         """
         return cls.values_eq(a, b)
 
-        #    def get_shape_info(self, obj):
-        """
-        Optional function. See TensorType().get_shape_info for definition.
-
-        """
-
-        #    def get_size(self, shape_info):
-        """
-        Optional function. See TensorType().get_size for definition.
-
-        """
-
 
 class CType(Type, CLinkerType):
     """Convenience wrapper combining `Type` and `CLinkerType`.
@@ -286,15 +274,16 @@ class CType(Type, CLinkerType):
 
 
 class Generic(CType, Singleton):
-    """
-    Represents a generic Python object.
+    r"""A type for a generic Python object exposed directly in C.
 
     This class implements the `CType` and `CLinkerType` interfaces
-    for generic PyObject instances.
+    for generic ``PyObject`` instances.
 
-    EXAMPLE of what this means, or when you would use this type.
+    It can be used to easily expose Python objects to `COp`\s.
 
-    WRITEME
+    For example, ``obj_const = Constant(Generic(), my_obj)`` will construct
+    an Aesara constant that can be used to access the Python object ``my_obj``
+    directly in a `COp` (e.g. as a ``PyObject *``).
 
     """
 
