@@ -6,7 +6,8 @@ import jax
 import jax.numpy as jnp
 import jax.scipy as jsp
 import numpy as np
-from numpy.random import RandomState
+from numpy.random import Generator, RandomState
+from numpy.random.bit_generator import _coerce_to_uint32_array
 
 from aesara.compile.ops import DeepCopyOp, ViewOp
 from aesara.configdefaults import config
@@ -15,8 +16,9 @@ from aesara.ifelse import IfElse
 from aesara.link.utils import fgraph_to_python
 from aesara.scalar import Softplus
 from aesara.scalar.basic import Cast, Clip, Composite, Identity, ScalarOp, Second
+from aesara.scalar.math import Erf, Erfc, Erfinv, Psi
 from aesara.scan.op import Scan
-from aesara.scan.utils import scan_args as ScanArgs
+from aesara.scan.utils import ScanArgs
 from aesara.tensor.basic import (
     Alloc,
     AllocDiag,
@@ -45,7 +47,7 @@ from aesara.tensor.extra_ops import (
 )
 from aesara.tensor.math import Dot, MaxAndArgmax
 from aesara.tensor.nlinalg import SVD, Det, Eig, Eigh, MatrixInverse, QRFull
-from aesara.tensor.nnet.basic import LogSoftmax, Softmax
+from aesara.tensor.nnet.basic import LogSoftmax, Softmax, SoftmaxGrad
 from aesara.tensor.random.op import RandomVariable
 from aesara.tensor.shape import Reshape, Shape, Shape_i, SpecifyShape
 from aesara.tensor.slinalg import Cholesky, Solve
@@ -105,6 +107,33 @@ def jax_typify_ndarray(data, dtype=None, **kwargs):
 def jax_typify_RandomState(state, **kwargs):
     state = state.get_state(legacy=False)
     state["bit_generator"] = numpy_bit_gens[state["bit_generator"]]
+    # XXX: Is this a reasonable approach?
+    state["jax_state"] = state["state"]["key"][0:2]
+    return state
+
+
+@jax_typify.register(Generator)
+def jax_typify_Generator(rng, **kwargs):
+    state = rng.__getstate__()
+    state["bit_generator"] = numpy_bit_gens[state["bit_generator"]]
+
+    # XXX: Is this a reasonable approach?
+    state["jax_state"] = _coerce_to_uint32_array(state["state"]["state"])[0:2]
+
+    # The "state" and "inc" values in a NumPy `Generator` are 128 bits, which
+    # JAX can't handle, so we split these values into arrays of 32 bit integers
+    # and then combine the first two into a single 64 bit integers.
+    #
+    # XXX: Depending on how we expect these values to be used, is this approach
+    # reasonable?
+    #
+    # TODO: We might as well remove these altogether, since this conversion
+    # should only occur once (e.g. when the graph is converted/JAX-compiled),
+    # and, from then on, we use the custom "jax_state" value.
+    inc_32 = _coerce_to_uint32_array(state["state"]["inc"])
+    state_32 = _coerce_to_uint32_array(state["state"]["state"])
+    state["state"]["inc"] = inc_32[0] << 32 | inc_32[1]
+    state["state"]["state"] = state_32[0] << 32 | state_32[1]
     return state
 
 
@@ -169,16 +198,31 @@ def jax_funcify_Identity(op, **kwargs):
 
 @jax_funcify.register(Softmax)
 def jax_funcify_Softmax(op, **kwargs):
+    axis = op.axis
+
     def softmax(x):
-        return jax.nn.softmax(x)
+        return jax.nn.softmax(x, axis=axis)
 
     return softmax
 
 
+@jax_funcify.register(SoftmaxGrad)
+def jax_funcify_SoftmaxGrad(op, **kwargs):
+    axis = op.axis
+
+    def softmax_grad(dy, sm):
+        dy_times_sm = dy * sm
+        return dy_times_sm - jnp.sum(dy_times_sm, axis=axis, keepdims=True) * sm
+
+    return softmax_grad
+
+
 @jax_funcify.register(LogSoftmax)
 def jax_funcify_LogSoftmax(op, **kwargs):
+    axis = op.axis
+
     def log_softmax(x):
-        return jax.nn.log_softmax(x)
+        return jax.nn.log_softmax(x, axis=axis)
 
     return log_softmax
 
@@ -666,7 +710,7 @@ def jax_funcify_Reshape(op, **kwargs):
 def jax_funcify_DimShuffle(op, **kwargs):
     def dimshuffle(x):
 
-        res = jnp.transpose(x, op.shuffle + op.drop)
+        res = jnp.transpose(x, op.transposition)
 
         shape = list(res.shape[: len(op.shuffle)])
 
@@ -698,12 +742,6 @@ def jax_funcify_Join(op, **kwargs):
             return tensors[view]
 
         else:
-            ndim = tensors[0].ndim
-            if axis < -ndim:
-                raise IndexError(
-                    f"Join axis {int(axis)} out of bounds [0, {int(ndim)})"
-                )
-
             return jnp.concatenate(tensors, axis=axis)
 
     return join
@@ -772,7 +810,7 @@ def jax_funcify_Cholesky(op, **kwargs):
 @jax_funcify.register(Solve)
 def jax_funcify_Solve(op, **kwargs):
 
-    if op.A_structure == "lower_triangular":
+    if op.assume_a != "gen" and op.lower:
         lower = True
     else:
         lower = False
@@ -999,7 +1037,7 @@ def jax_funcify_BatchedDot(op, **kwargs):
 
 
 @jax_funcify.register(RandomVariable)
-def jax_funcify_RandomVariable(op, **kwargs):
+def jax_funcify_RandomVariable(op, node, **kwargs):
     name = op.name
 
     if not hasattr(jax.random, name):
@@ -1007,13 +1045,68 @@ def jax_funcify_RandomVariable(op, **kwargs):
             f"No JAX conversion for the given distribution: {name}"
         )
 
-    def random_variable(rng, size, dtype, *args):
-        prng = jax.random.PRNGKey(rng["state"]["key"][0])
-        dtype = jnp.dtype(dtype)
+    dtype = node.outputs[1].dtype
+
+    def random_variable(rng, size, dtype_num, *args):
+        if not op.inplace:
+            rng = rng.copy()
+        prng = rng["jax_state"]
         data = getattr(jax.random, name)(key=prng, shape=size)
         smpl_value = jnp.array(data, dtype=dtype)
-        prng = jax.random.split(prng, num=1)[0]
-        jax.ops.index_update(rng["state"]["key"], 0, prng[0])
+        rng["jax_state"] = jax.random.split(prng, num=1)[0]
         return (rng, smpl_value)
 
     return random_variable
+
+
+@jax_funcify.register(Erf)
+def jax_funcify_Erf(op, node, **kwargs):
+    def erf(x):
+        return jax.scipy.special.erf(x)
+
+    return erf
+
+
+@jax_funcify.register(Erfc)
+def jax_funcify_Erfc(op, **kwargs):
+    def erfc(x):
+        return jax.scipy.special.erfc(x)
+
+    return erfc
+
+
+# Commented out because jax.scipy does not have erfcx,
+# but leaving the implementation in here just in case we ever see
+# a JAX implementation of Erfcx.
+# See https://github.com/google/jax/issues/1987 for context.
+# @jax_funcify.register(Erfcx)
+# def jax_funcify_Erfcx(op, **kwargs):
+#     def erfcx(x):
+#         return jax.scipy.special.erfcx(x)
+#     return erfcx
+
+
+@jax_funcify.register(Erfinv)
+def jax_funcify_Erfinv(op, **kwargs):
+    def erfinv(x):
+        return jax.scipy.special.erfinv(x)
+
+    return erfinv
+
+
+# Commented out because jax.scipy does not have Erfcinv,
+# but leaving the implementation in here just in case we ever see
+# a JAX implementation of Erfcinv.
+# @jax_funcify.register(Erfcinv)
+# def jax_funcify_Erfcinv(op, **kwargs):
+#     def erfcinv(x):
+#         return jax.scipy.special.erfcinv(x)
+#     return erfcinv
+
+
+@jax_funcify.register(Psi)
+def jax_funcify_Psi(op, node, **kwargs):
+    def psi(x):
+        return jax.scipy.special.digamma(x)
+
+    return psi
