@@ -5,19 +5,26 @@ As SciPy is not always available, we treat them separately.
 """
 
 import os
+import warnings
 
 import numpy as np
+import scipy.special
+import scipy.stats
 
 from aesara.configdefaults import config
 from aesara.gradient import grad_not_implemented
 from aesara.scalar.basic import (
     BinaryScalarOp,
+    ScalarOp,
     UnaryScalarOp,
     complex_types,
     discrete_types,
     exp,
     float64,
     float_types,
+    log,
+    log1p,
+    true_div,
     upcast,
     upgrade_to_float,
     upgrade_to_float64,
@@ -25,26 +32,11 @@ from aesara.scalar.basic import (
 )
 
 
-imported_scipy_special = False
-try:
-    import scipy.special
-    import scipy.stats
-
-    imported_scipy_special = True
-# Importing scipy.special may raise ValueError.
-# See http://projects.scipy.org/scipy/ticket/1739
-except (ImportError, ValueError):
-    pass
-
-
 class Erf(UnaryScalarOp):
     nfunc_spec = ("scipy.special.erf", 1, 1)
 
     def impl(self, x):
-        if imported_scipy_special:
-            return scipy.special.erf(x)
-        else:
-            super().impl(x)
+        return scipy.special.erf(x)
 
     def L_op(self, inputs, outputs, grads):
         (x,) = inputs
@@ -78,10 +70,7 @@ class Erfc(UnaryScalarOp):
     nfunc_spec = ("scipy.special.erfc", 1, 1)
 
     def impl(self, x):
-        if imported_scipy_special:
-            return scipy.special.erfc(x)
-        else:
-            super().impl(x)
+        return scipy.special.erfc(x)
 
     def L_op(self, inputs, outputs, grads):
         (x,) = inputs
@@ -130,10 +119,7 @@ class Erfcx(UnaryScalarOp):
     nfunc_spec = ("scipy.special.erfcx", 1, 1)
 
     def impl(self, x):
-        if imported_scipy_special:
-            return scipy.special.erfcx(x)
-        else:
-            super().impl(x)
+        return scipy.special.erfcx(x)
 
     def L_op(self, inputs, outputs, grads):
         (x,) = inputs
@@ -195,10 +181,7 @@ class Erfinv(UnaryScalarOp):
     nfunc_spec = ("scipy.special.erfinv", 1, 1)
 
     def impl(self, x):
-        if imported_scipy_special:
-            return scipy.special.erfinv(x)
-        else:
-            super().impl(x)
+        return scipy.special.erfinv(x)
 
     def L_op(self, inputs, outputs, grads):
         (x,) = inputs
@@ -232,10 +215,7 @@ class Erfcinv(UnaryScalarOp):
     nfunc_spec = ("scipy.special.erfcinv", 1, 1)
 
     def impl(self, x):
-        if imported_scipy_special:
-            return scipy.special.erfcinv(x)
-        else:
-            super().impl(x)
+        return scipy.special.erfcinv(x)
 
     def L_op(self, inputs, outputs, grads):
         (x,) = inputs
@@ -273,10 +253,7 @@ class Gamma(UnaryScalarOp):
         return scipy.special.gamma(x)
 
     def impl(self, x):
-        if imported_scipy_special:
-            return Gamma.st_impl(x)
-        else:
-            super().impl(x)
+        return Gamma.st_impl(x)
 
     def L_op(self, inputs, outputs, gout):
         (x,) = inputs
@@ -315,10 +292,7 @@ class GammaLn(UnaryScalarOp):
         return scipy.special.gammaln(x)
 
     def impl(self, x):
-        if imported_scipy_special:
-            return GammaLn.st_impl(x)
-        else:
-            super().impl(x)
+        return GammaLn.st_impl(x)
 
     def L_op(self, inputs, outputs, grads):
         (x,) = inputs
@@ -362,10 +336,7 @@ class Psi(UnaryScalarOp):
         return scipy.special.psi(x)
 
     def impl(self, x):
-        if imported_scipy_special:
-            return Psi.st_impl(x)
-        else:
-            super().impl(x)
+        return Psi.st_impl(x)
 
     def L_op(self, inputs, outputs, grads):
         (x,) = inputs
@@ -456,10 +427,7 @@ class TriGamma(UnaryScalarOp):
         return scipy.special.polygamma(1, x)
 
     def impl(self, x):
-        if imported_scipy_special:
-            return TriGamma.st_impl(x)
-        else:
-            super().impl(x)
+        return TriGamma.st_impl(x)
 
     def grad(self, inputs, outputs_gradients):
         raise NotImplementedError()
@@ -545,10 +513,7 @@ class Chi2SF(BinaryScalarOp):
         return scipy.stats.chi2.sf(x, k)
 
     def impl(self, x, k):
-        if imported_scipy_special:
-            return Chi2SF.st_impl(x, k)
-        else:
-            super().impl(x, k)
+        return Chi2SF.st_impl(x, k)
 
     def c_support_code(self, **kwargs):
         with open(os.path.join(os.path.dirname(__file__), "c_code", "gamma.c")) as f:
@@ -589,10 +554,15 @@ class GammaInc(BinaryScalarOp):
         return scipy.special.gammainc(k, x)
 
     def impl(self, k, x):
-        if imported_scipy_special:
-            return GammaInc.st_impl(k, x)
-        else:
-            super().impl(k, x)
+        return GammaInc.st_impl(k, x)
+
+    def grad(self, inputs, grads):
+        (k, x) = inputs
+        (gz,) = grads
+        return [
+            gz * gammainc_der(k, x),
+            gz * exp(-x + (k - 1) * log(x) - gammaln(k)),
+        ]
 
     def c_support_code(self, **kwargs):
         with open(os.path.join(os.path.dirname(__file__), "c_code", "gamma.c")) as f:
@@ -630,13 +600,18 @@ class GammaIncC(BinaryScalarOp):
 
     @staticmethod
     def st_impl(k, x):
-        return scipy.special.gammaincc(x, k)
+        return scipy.special.gammaincc(k, x)
 
     def impl(self, k, x):
-        if imported_scipy_special:
-            return GammaIncC.st_impl(k, x)
-        else:
-            super().impl(k, x)
+        return GammaIncC.st_impl(k, x)
+
+    def grad(self, inputs, grads):
+        (k, x) = inputs
+        (gz,) = grads
+        return [
+            gz * gammaincc_der(k, x),
+            gz * -exp(-x + (k - 1) * log(x) - gammaln(k)),
+        ]
 
     def c_support_code(self, **kwargs):
         with open(os.path.join(os.path.dirname(__file__), "c_code", "gamma.c")) as f:
@@ -665,6 +640,159 @@ class GammaIncC(BinaryScalarOp):
 gammaincc = GammaIncC(upgrade_to_float, name="gammaincc")
 
 
+class GammaIncDer(BinaryScalarOp):
+    """
+    Gradient of the the regularized lower gamma function (P) wrt to the first
+    argument (k, a.k.a. alpha). Adapted from STAN `grad_reg_lower_inc_gamma.hpp`
+
+    Reference: Gautschi, W. (1979). A computational procedure for incomplete gamma functions.
+    ACM Transactions on Mathematical Software (TOMS), 5(4), 466-481.
+    """
+
+    def impl(self, k, x):
+
+        if x == 0:
+            return 0
+
+        sqrt_exp = -756 - x ** 2 + 60 * x
+        if (
+            (k < 0.8 and x > 15)
+            or (k < 12 and x > 30)
+            or (sqrt_exp > 0 and k < np.sqrt(sqrt_exp))
+        ):
+            return -GammaIncCDer.st_impl(k, x)
+
+        precision = 1e-10
+        max_iters = int(1e5)
+
+        log_x = np.log(x)
+        log_gamma_k_plus_1 = scipy.special.gammaln(k + 1)
+
+        k_plus_n = k
+        log_gamma_k_plus_n_plus_1 = log_gamma_k_plus_1
+        sum_a = 0.0
+        for n in range(0, max_iters + 1):
+            term = np.exp(k_plus_n * log_x - log_gamma_k_plus_n_plus_1)
+            sum_a += term
+
+            if term <= precision:
+                break
+
+            log_gamma_k_plus_n_plus_1 += np.log1p(k_plus_n)
+            k_plus_n += 1
+
+        if n >= max_iters:
+            warnings.warn(
+                f"gammainc_der did not converge after {n} iterations",
+                RuntimeWarning,
+            )
+            return np.nan
+
+        k_plus_n = k
+        log_gamma_k_plus_n_plus_1 = log_gamma_k_plus_1
+        sum_b = 0.0
+        for n in range(0, max_iters + 1):
+            term = np.exp(
+                k_plus_n * log_x - log_gamma_k_plus_n_plus_1
+            ) * scipy.special.digamma(k_plus_n + 1)
+            sum_b += term
+
+            if term <= precision and n >= 1:  # Require at least two iterations
+                return np.exp(-x) * (log_x * sum_a - sum_b)
+
+            log_gamma_k_plus_n_plus_1 += np.log1p(k_plus_n)
+            k_plus_n += 1
+
+        warnings.warn(
+            f"gammainc_der did not converge after {n} iterations",
+            RuntimeWarning,
+        )
+        return np.nan
+
+
+gammainc_der = GammaIncDer(upgrade_to_float, name="gammainc_der")
+
+
+class GammaIncCDer(BinaryScalarOp):
+    """
+    Gradient of the the regularized upper gamma function (Q) wrt to the first
+    argument (k, a.k.a. alpha). Adapted from STAN `grad_reg_inc_gamma.hpp`
+    """
+
+    @staticmethod
+    def st_impl(k, x):
+        gamma_k = scipy.special.gamma(k)
+        digamma_k = scipy.special.digamma(k)
+        log_x = np.log(x)
+
+        # asymptotic expansion http://dlmf.nist.gov/8.11#E2
+        if (x >= k) and (x >= 8):
+            S = 0
+            k_minus_one_minus_n = k - 1
+            fac = k_minus_one_minus_n
+            dfac = 1
+            xpow = x
+            delta = dfac / xpow
+
+            for n in range(1, 10):
+                k_minus_one_minus_n -= 1
+                S += delta
+                xpow *= x
+                dfac = k_minus_one_minus_n * dfac + fac
+                fac *= k_minus_one_minus_n
+                delta = dfac / xpow
+                if np.isinf(delta):
+                    warnings.warn(
+                        "gammaincc_der did not converge",
+                        RuntimeWarning,
+                    )
+                    return np.nan
+
+            return (
+                scipy.special.gammaincc(k, x) * (log_x - digamma_k)
+                + np.exp(-x + (k - 1) * log_x) * S / gamma_k
+            )
+
+        # gradient of series expansion http://dlmf.nist.gov/8.7#E3
+        else:
+            log_precision = np.log(1e-6)
+            max_iters = int(1e5)
+            S = 0
+            log_s = 0.0
+            s_sign = 1
+            log_delta = log_s - 2 * np.log(k)
+            for n in range(1, max_iters + 1):
+                S += np.exp(log_delta) if s_sign > 0 else -np.exp(log_delta)
+                s_sign = -s_sign
+                log_s += log_x - np.log(n)
+                log_delta = log_s - 2 * np.log(n + k)
+
+                if np.isinf(log_delta):
+                    warnings.warn(
+                        "gammaincc_der did not converge",
+                        RuntimeWarning,
+                    )
+                    return np.nan
+
+                if log_delta <= log_precision:
+                    return (
+                        scipy.special.gammainc(k, x) * (digamma_k - log_x)
+                        + np.exp(k * log_x) * S / gamma_k
+                    )
+
+            warnings.warn(
+                f"gammaincc_der did not converge after {n} iterations",
+                RuntimeWarning,
+            )
+            return np.nan
+
+    def impl(self, k, x):
+        return self.st_impl(k, x)
+
+
+gammaincc_der = GammaIncCDer(upgrade_to_float, name="gammaincc_der")
+
+
 class GammaU(BinaryScalarOp):
     """
     compute the upper incomplete gamma function.
@@ -677,10 +805,7 @@ class GammaU(BinaryScalarOp):
         return scipy.special.gammaincc(k, x) * scipy.special.gamma(k)
 
     def impl(self, k, x):
-        if imported_scipy_special:
-            return GammaU.st_impl(k, x)
-        else:
-            super().impl(k, x)
+        return GammaU.st_impl(k, x)
 
     def c_support_code(self, **kwargs):
         with open(os.path.join(os.path.dirname(__file__), "c_code", "gamma.c")) as f:
@@ -721,10 +846,7 @@ class GammaL(BinaryScalarOp):
         return scipy.special.gammainc(k, x) * scipy.special.gamma(k)
 
     def impl(self, k, x):
-        if imported_scipy_special:
-            return GammaL.st_impl(k, x)
-        else:
-            super().impl(k, x)
+        return GammaL.st_impl(k, x)
 
     def c_support_code(self, **kwargs):
         with open(os.path.join(os.path.dirname(__file__), "c_code", "gamma.c")) as f:
@@ -765,10 +887,7 @@ class Jv(BinaryScalarOp):
         return scipy.special.jv(v, x)
 
     def impl(self, v, x):
-        if imported_scipy_special:
-            return self.st_impl(v, x)
-        else:
-            super().impl(v, x)
+        return self.st_impl(v, x)
 
     def grad(self, inputs, grads):
         v, x = inputs
@@ -794,10 +913,7 @@ class J1(UnaryScalarOp):
         return scipy.special.j1(x)
 
     def impl(self, x):
-        if imported_scipy_special:
-            return self.st_impl(x)
-        else:
-            super().impl(x)
+        return self.st_impl(x)
 
     def grad(self, inputs, grads):
         (x,) = inputs
@@ -828,10 +944,7 @@ class J0(UnaryScalarOp):
         return scipy.special.j0(x)
 
     def impl(self, x):
-        if imported_scipy_special:
-            return self.st_impl(x)
-        else:
-            super().impl(x)
+        return self.st_impl(x)
 
     def grad(self, inp, grads):
         (x,) = inp
@@ -862,10 +975,7 @@ class Iv(BinaryScalarOp):
         return scipy.special.iv(v, x)
 
     def impl(self, v, x):
-        if imported_scipy_special:
-            return self.st_impl(v, x)
-        else:
-            super().impl(v, x)
+        return self.st_impl(v, x)
 
     def grad(self, inputs, grads):
         v, x = inputs
@@ -891,10 +1001,7 @@ class I1(UnaryScalarOp):
         return scipy.special.i1(x)
 
     def impl(self, x):
-        if imported_scipy_special:
-            return self.st_impl(x)
-        else:
-            super().impl(x)
+        return self.st_impl(x)
 
     def grad(self, inputs, grads):
         (x,) = inputs
@@ -917,10 +1024,7 @@ class I0(UnaryScalarOp):
         return scipy.special.i0(x)
 
     def impl(self, x):
-        if imported_scipy_special:
-            return self.st_impl(x)
-        else:
-            super().impl(x)
+        return self.st_impl(x)
 
     def grad(self, inp, grads):
         (x,) = inp
@@ -939,10 +1043,7 @@ class Sigmoid(UnaryScalarOp):
     nfunc_spec = ("scipy.special.expit", 1, 1)
 
     def impl(self, x):
-        if imported_scipy_special:
-            return scipy.special.expit(x)
-        else:
-            super().impl(x)
+        return scipy.special.expit(x)
 
     def grad(self, inp, grads):
         (x,) = inp
@@ -1070,3 +1171,264 @@ class Softplus(UnaryScalarOp):
 
 
 softplus = Softplus(upgrade_to_float, name="scalar_softplus")
+
+
+class Log1mexp(UnaryScalarOp):
+    r"""
+    Compute log(1 - exp(x)), also known as log1mexp
+
+    This function is numerically more stable than the naive approach.
+
+    For details, see
+    https://cran.r-project.org/web/packages/Rmpfr/vignettes/log1mexp-note.pdf
+
+    References
+    ----------
+    .. [Machler2012] Martin Mächler (2012).
+        "Accurately computing `\log(1-\exp(- \mid a \mid))` Assessed by the Rmpfr package"
+    """
+
+    @staticmethod
+    def static_impl(x):
+        if x < np.log(0.5):
+            return np.log1p(-np.exp(x))
+        else:
+            return np.log(-np.expm1(x))
+
+    def impl(self, x):
+        return Log1mexp.static_impl(x)
+
+    def grad(self, inp, grads):
+        (x,) = inp
+        (gz,) = grads
+        return [gz * true_div(1.0, 1.0 - exp(-x))]
+
+    def c_code(self, node, name, inp, out, sub):
+        (x,) = inp
+        (z,) = out
+
+        if node.inputs[0].type in float_types:
+            if node.inputs[0].type == float64:
+                return f"{z} = {x} < -0.6931471805599453 ? log1p(-exp({x})) : log(-expm1({x}));"
+            else:
+                return f"{z} = {x} < -0.6931471805599453f ? log1p(-exp({x})) : log(-expm1({x}));"
+        else:
+            raise NotImplementedError("only floating point is implemented")
+
+
+log1mexp = Log1mexp(upgrade_to_float, name="scalar_log1mexp")
+
+
+class BetaInc(ScalarOp):
+    """
+    Regularized incomplete beta function
+    """
+
+    nin = 3
+    nfunc_spec = ("scipy.special.betainc", 3, 1)
+
+    def impl(self, a, b, x):
+        return scipy.special.betainc(a, b, x)
+
+    def grad(self, inp, grads):
+        a, b, x = inp
+        (gz,) = grads
+
+        return [
+            gz * betainc_der(a, b, x, True),
+            gz * betainc_der(a, b, x, False),
+            gz
+            * exp(
+                log1p(-x) * (b - 1)
+                + log(x) * (a - 1)
+                - (gammaln(a) + gammaln(b) - gammaln(a + b))
+            ),
+        ]
+
+
+betainc = BetaInc(upgrade_to_float_no_complex, name="betainc")
+
+
+class BetaIncDer(ScalarOp):
+    """
+    Gradient of the regularized incomplete beta function wrt to the first
+    argument (alpha) or the second argument (beta), depending on whether the
+    fourth argument to betainc_der is `True` or `False`, respectively.
+
+    Reference: Boik, R. J., & Robison-Cox, J. F. (1998). Derivatives of the incomplete beta function.
+    Journal of Statistical Software, 3(1), 1-20.
+    """
+
+    nin = 4
+
+    def impl(self, p, q, x, wrtp):
+        def _betainc_a_n(f, p, q, n):
+            """
+            Numerator (a_n) of the nth approximant of the continued fraction
+            representation of the regularized incomplete beta function
+            """
+
+            if n == 1:
+                return p * f * (q - 1) / (q * (p + 1))
+
+            p2n = p + 2 * n
+            F1 = p ** 2 * f ** 2 * (n - 1) / (q ** 2)
+            F2 = (
+                (p + q + n - 2)
+                * (p + n - 1)
+                * (q - n)
+                / ((p2n - 3) * (p2n - 2) ** 2 * (p2n - 1))
+            )
+
+            return F1 * F2
+
+        def _betainc_b_n(f, p, q, n):
+            """
+            Offset (b_n) of the nth approximant of the continued fraction
+            representation of the regularized incomplete beta function
+            """
+            pf = p * f
+            p2n = p + 2 * n
+
+            N1 = 2 * (pf + 2 * q) * n * (n + p - 1) + p * q * (p - 2 - pf)
+            D1 = q * (p2n - 2) * p2n
+
+            return N1 / D1
+
+        def _betainc_da_n_dp(f, p, q, n):
+            """
+            Derivative of a_n wrt p
+            """
+
+            if n == 1:
+                return -p * f * (q - 1) / (q * (p + 1) ** 2)
+
+            pp = p ** 2
+            ppp = pp * p
+            p2n = p + 2 * n
+
+            N1 = -(n - 1) * f ** 2 * pp * (q - n)
+            N2a = (-8 + 8 * p + 8 * q) * n ** 3
+            N2b = (16 * pp + (-44 + 20 * q) * p + 26 - 24 * q) * n ** 2
+            N2c = (10 * ppp + (14 * q - 46) * pp + (-40 * q + 66) * p - 28 + 24 * q) * n
+            N2d = 2 * pp ** 2 + (-13 + 3 * q) * ppp + (-14 * q + 30) * pp
+            N2e = (-29 + 19 * q) * p + 10 - 8 * q
+
+            D1 = q ** 2 * (p2n - 3) ** 2
+            D2 = (p2n - 2) ** 3 * (p2n - 1) ** 2
+
+            return (N1 / D1) * (N2a + N2b + N2c + N2d + N2e) / D2
+
+        def _betainc_da_n_dq(f, p, q, n):
+            """
+            Derivative of a_n wrt q
+            """
+            if n == 1:
+                return p * f / (q * (p + 1))
+
+            p2n = p + 2 * n
+            F1 = (p ** 2 * f ** 2 / (q ** 2)) * (n - 1) * (p + n - 1) * (2 * q + p - 2)
+            D1 = (p2n - 3) * (p2n - 2) ** 2 * (p2n - 1)
+
+            return F1 / D1
+
+        def _betainc_db_n_dp(f, p, q, n):
+            """
+            Derivative of b_n wrt p
+            """
+            p2n = p + 2 * n
+            pp = p ** 2
+            q4 = 4 * q
+            p4 = 4 * p
+
+            F1 = (p * f / q) * (
+                (-p4 - q4 + 4) * n ** 2 + (p4 - 4 + q4 - 2 * pp) * n + pp * q
+            )
+            D1 = (p2n - 2) ** 2 * p2n ** 2
+
+            return F1 / D1
+
+        def _betainc_db_n_dq(f, p, q, n):
+            """
+            Derivative of b_n wrt to q
+            """
+            p2n = p + 2 * n
+            return -(p ** 2 * f) / (q * (p2n - 2) * p2n)
+
+        # Input validation
+        if not (0 <= x <= 1) or p < 0 or q < 0:
+            return np.nan
+
+        if x > (p / (p + q)):
+            return -self.impl(q, p, 1 - x, not wrtp)
+
+        min_iters = 3
+        max_iters = 200
+        err_threshold = 1e-12
+
+        derivative_old = 0
+
+        Am2, Am1 = 1, 1
+        Bm2, Bm1 = 0, 1
+        dAm2, dAm1 = 0, 0
+        dBm2, dBm1 = 0, 0
+
+        f = (q * x) / (p * (1 - x))
+        K = np.exp(
+            p * np.log(x)
+            + (q - 1) * np.log1p(-x)
+            - np.log(p)
+            - scipy.special.betaln(p, q)
+        )
+        if wrtp:
+            dK = (
+                np.log(x)
+                - 1 / p
+                + scipy.special.digamma(p + q)
+                - scipy.special.digamma(p)
+            )
+        else:
+            dK = np.log1p(-x) + scipy.special.digamma(p + q) - scipy.special.digamma(q)
+
+        for n in range(1, max_iters + 1):
+            a_n_ = _betainc_a_n(f, p, q, n)
+            b_n_ = _betainc_b_n(f, p, q, n)
+            if wrtp:
+                da_n = _betainc_da_n_dp(f, p, q, n)
+                db_n = _betainc_db_n_dp(f, p, q, n)
+            else:
+                da_n = _betainc_da_n_dq(f, p, q, n)
+                db_n = _betainc_db_n_dq(f, p, q, n)
+
+            A = a_n_ * Am2 + b_n_ * Am1
+            B = a_n_ * Bm2 + b_n_ * Bm1
+            dA = da_n * Am2 + a_n_ * dAm2 + db_n * Am1 + b_n_ * dAm1
+            dB = da_n * Bm2 + a_n_ * dBm2 + db_n * Bm1 + b_n_ * dBm1
+
+            Am2, Am1 = Am1, A
+            Bm2, Bm1 = Bm1, B
+            dAm2, dAm1 = dAm1, dA
+            dBm2, dBm1 = dBm1, dB
+
+            if n < min_iters - 1:
+                continue
+
+            F1 = A / B
+            F2 = (dA - F1 * dB) / B
+            derivative = K * (F1 * dK + F2)
+
+            errapx = abs(derivative_old - derivative)
+            d_errapx = errapx / max(err_threshold, abs(derivative))
+            derivative_old = derivative
+
+            if d_errapx <= err_threshold:
+                return derivative
+
+        warnings.warn(
+            f"betainc_der did not converge after {n} iterations",
+            RuntimeWarning,
+        )
+        return np.nan
+
+
+betainc_der = BetaIncDer(upgrade_to_float_no_complex, name="betainc_der")
