@@ -14,13 +14,15 @@ from aesara.graph.utils import MethodNotDefined
 from aesara.link.c.basic import failure_code
 from aesara.misc.frozendict import frozendict
 from aesara.misc.safe_asarray import _asarray
-from aesara.printing import FunctionPrinter, pprint
+from aesara.printing import FunctionPrinter, Printer, pprint
 from aesara.scalar import get_scalar_type
 from aesara.scalar.basic import Scalar
 from aesara.scalar.basic import bool as scalar_bool
 from aesara.scalar.basic import identity as scalar_identity
 from aesara.scalar.basic import transfer_type, upcast
+from aesara.tensor import _get_vector_length, as_tensor_variable
 from aesara.tensor import elemwise_cgen as cgen
+from aesara.tensor import get_vector_length
 from aesara.tensor.type import (
     TensorType,
     continuous_dtypes,
@@ -117,47 +119,27 @@ class DimShuffle(ExternalCOp):
 
     @property
     def params_type(self):
-        # We can't directly create `params_type` as class attribute
-        # because of importation issues related to TensorType.
         return ParamsType(
-            input_broadcastable=TensorType(dtype="bool", broadcastable=(False,)),
-            _new_order=lvector,
-            transposition=TensorType(dtype="uint32", broadcastable=(False,)),
+            shuffle=lvector,
+            augment=lvector,
+            transposition=lvector,
             inplace=scalar_bool,
         )
 
-    @property
-    def _new_order(self):
-        # Param for C code.
-        # self.new_order may contain 'x', which is not a valid integer value.
-        # We replace it with -1.
-        return [(-1 if x == "x" else x) for x in self.new_order]
-
-    @property
-    def transposition(self):
-        return self.shuffle + self.drop
-
-    def __init__(self, input_broadcastable, new_order, inplace=True):
+    def __init__(self, input_broadcastable, new_order):
         super().__init__([self.c_func_file], self.c_func_name)
+
         self.input_broadcastable = tuple(input_broadcastable)
         self.new_order = tuple(new_order)
-        if inplace is True:
-            self.inplace = inplace
-        else:
-            raise ValueError(
-                "DimShuffle is inplace by default and hence the inplace for DimShuffle must be true"
-            )
+
+        self.inplace = True
 
         for i, j in enumerate(new_order):
             if j != "x":
-                # There is a bug in numpy that results in
-                # isinstance(x, integer_types) returning False for
-                # numpy integers.  See
-                # <http://projects.scipy.org/numpy/ticket/2235>.
                 if not isinstance(j, (int, np.integer)):
                     raise TypeError(
-                        "DimShuffle indices must be python ints. "
-                        f"Got: '{j}' of type '{type(j)}'."
+                        "DimShuffle indices must be Python ints; got "
+                        f"{j} of type {type(j)}."
                     )
                 if j >= len(input_broadcastable):
                     raise ValueError(
@@ -167,31 +149,30 @@ class DimShuffle(ExternalCOp):
                 if j in new_order[(i + 1) :]:
                     raise ValueError(
                         "The same input dimension may not appear "
-                        "twice in the list of output dimensions",
-                        new_order,
+                        f"twice in the list of output dimensions: {new_order}"
                     )
 
-        # list of dimensions of the input to drop
-        self.drop = []
+        # List of input dimensions to drop
+        drop = []
         for i, b in enumerate(input_broadcastable):
             if i not in new_order:
-                # we want to drop this dimension because it's not a value in
-                # new_order
-                if b == 1:  # 1 aka True
-                    self.drop.append(i)
+                # We want to drop this dimension because it's not a value in
+                # `new_order`
+                if b == 1:
+                    drop.append(i)
                 else:
-                    # we cannot drop non-broadcastable dimensions
+                    # We cannot drop non-broadcastable dimensions
                     raise ValueError(
-                        "You cannot drop a non-broadcastable dimension.",
-                        (input_broadcastable, new_order),
+                        "Cannot drop a non-broadcastable dimension: "
+                        f"{input_broadcastable}, {new_order}"
                     )
 
-        # this is the list of the original dimensions that we keep
+        # This is the list of the original dimensions that we keep
         self.shuffle = [x for x in new_order if x != "x"]
-
-        # list of dimensions of the output that are broadcastable and were not
+        self.transposition = self.shuffle + drop
+        # List of dimensions of the output that are broadcastable and were not
         # in the original input
-        self.augment = [i for i, x in enumerate(new_order) if x == "x"]
+        self.augment = sorted([i for i, x in enumerate(new_order) if x == "x"])
 
         if self.inplace:
             self.view_map = {0: [0]}
@@ -204,7 +185,7 @@ class DimShuffle(ExternalCOp):
             super().__init__([self.c_func_file], self.c_func_name)
 
     def make_node(self, _input):
-        input = aesara.tensor.basic.as_tensor_variable(_input)
+        input = as_tensor_variable(_input)
         ib = tuple(input.type.broadcastable)
         if not ib == self.input_broadcastable:
             if len(ib) != len(self.input_broadcastable):
@@ -239,27 +220,23 @@ class DimShuffle(ExternalCOp):
             return "DimShuffle{%s}" % ",".join(str(x) for x in self.new_order)
 
     def perform(self, node, inp, out, params):
-        (input,) = inp
+        (res,) = inp
         (storage,) = out
-        # drop
-        res = input
+
         if type(res) != np.ndarray and type(res) != np.memmap:
             raise TypeError(res)
 
-        # transpose
-        res = res.transpose(self.shuffle + self.drop)
+        res = res.transpose(self.transposition)
 
-        # augment
         shape = list(res.shape[: len(self.shuffle)])
         for augm in self.augment:
             shape.insert(augm, 1)
         res = res.reshape(shape)
 
-        # copy (if not inplace)
         if not self.inplace:
             res = np.copy(res)
 
-        storage[0] = np.asarray(res)  # asarray puts scalars back into array
+        storage[0] = np.asarray(res)
 
     def infer_shape(self, fgraph, node, shapes):
         (ishp,) = shapes
@@ -274,10 +251,9 @@ class DimShuffle(ExternalCOp):
     def R_op(self, inputs, eval_points):
         if None in eval_points:
             return [None]
-        return self(*eval_points, **dict(return_list=True))
+        return self(*eval_points, return_list=True)
 
     def grad(self, inp, grads):
-        from aesara.tensor.basic import as_tensor_variable
 
         (x,) = inp
         (gz,) = grads
@@ -299,7 +275,7 @@ class DimShuffle(ExternalCOp):
             ]
 
 
-class DimShufflePrinter:
+class DimShufflePrinter(Printer):
     def __p(self, new_order, pstate, r):
         if new_order != () and new_order[0] == "x":
             return f"{self.__p(new_order[1:], pstate, r)}"
@@ -482,7 +458,7 @@ second dimension
         is left-completed to the greatest number of dimensions with 1s
         using DimShuffle.
         """
-        inputs = list(map(aesara.tensor.basic.as_tensor_variable, inputs))
+        inputs = [as_tensor_variable(i) for i in inputs]
         out_dtypes, out_broadcastables, inputs = self.get_output_info(
             DimShuffle, *inputs
         )
@@ -504,7 +480,7 @@ second dimension
             return self.name
 
     def R_op(self, inputs, eval_points):
-        outs = self(*inputs, **dict(return_list=True))
+        outs = self(*inputs, return_list=True)
         rval = [None for x in outs]
         # For each output
         for idx, out in enumerate(outs):
@@ -1145,7 +1121,7 @@ second dimension
                                 % locals()
                             )
                     if self.openmp:
-                        contig += """#pragma omp parallel for if(n>={int(config.openmp_elemwise_minsize)})
+                        contig += f"""#pragma omp parallel for if(n>={int(config.openmp_elemwise_minsize)})
                         """
                     contig += (
                         """
@@ -1313,8 +1289,6 @@ class CAReduce(COp):
         return input_dtype
 
     def make_node(self, input):
-        from aesara.tensor.basic import as_tensor_variable
-
         input = as_tensor_variable(input)
         inp_dims = input.type.ndim
         inp_bdcast = input.type.broadcastable
@@ -1758,7 +1732,7 @@ class CAReduceDtype(CAReduce):
         # We need to redefine make_node so that, if self.dtype is None,
         # we can infer what dtype should be, and create a node from an Op
         # of the appropriate dtype.
-        input = aesara.tensor.basic.as_tensor_variable(input)
+        input = as_tensor_variable(input)
         dtype = self._output_dtype(input.dtype)
         acc_dtype = self._acc_dtype(input.dtype)
 
@@ -1834,7 +1808,7 @@ def scalar_elemwise(*symbol, nfunc=None, nin=None, nout=None, symbolname=None):
         rval.__epydoc_asRoutine = symbol
         rval.__module__ = symbol.__module__
 
-        pprint.assign(rval, FunctionPrinter(symbolname.replace("_inplace", "=")))
+        pprint.assign(rval, FunctionPrinter([symbolname.replace("_inplace", "=")]))
 
         return rval
 
@@ -1842,3 +1816,11 @@ def scalar_elemwise(*symbol, nfunc=None, nin=None, nout=None, symbolname=None):
         return construct(symbol[0])
     else:
         return construct
+
+
+@_get_vector_length.register(Elemwise)
+def _get_vector_length_Elemwise(op, var):
+    if len(var.owner.inputs) == 1 and len(var.owner.outputs) == 1:
+        return get_vector_length(var.owner.inputs[0])
+
+    raise ValueError(f"Length of {var} cannot be determined")
