@@ -1348,6 +1348,9 @@ def psi(a):
     """derivative of log gamma function"""
 
 
+digamma = psi
+
+
 @scalar_elemwise
 def tri_gamma(a):
     """second derivative of the log gamma function"""
@@ -1421,6 +1424,19 @@ def softplus(x):
     """Compute log(1 + exp(x)), also known as softplus or log1pexp"""
 
 
+log1pexp = softplus
+
+
+@scalar_elemwise
+def log1mexp(x):
+    """Compute log(1 - exp(x)), also known as log1mexp"""
+
+
+@scalar_elemwise
+def betainc(a, b, x):
+    """Regularized incomplete beta function"""
+
+
 @scalar_elemwise
 def real(z):
     """Return real component of complex-valued tensor `z`"""
@@ -2449,7 +2465,7 @@ class Sum(CAReduceDtype):
         # part of self
         if None in eval_points:
             return [None]
-        return self(*eval_points, **dict(return_list=True))
+        return self(*eval_points, return_list=True)
 
 
 def sum(input, axis=None, dtype=None, keepdims=False, acc_dtype=None):
@@ -2479,7 +2495,7 @@ def sum(input, axis=None, dtype=None, keepdims=False, acc_dtype=None):
     return out
 
 
-pprint.assign(Sum(), printing.FunctionPrinter("sum"))
+pprint.assign(Sum, printing.FunctionPrinter(["sum"], ["axis"]))
 
 
 class Prod(CAReduceDtype):
@@ -2764,6 +2780,53 @@ def power(x, y):
     return x ** y
 
 
+def logaddexp(*xs):
+    """Logarithm of the sum of exponentiations of the inputs.
+
+    See ``numpy.logaddexp``.
+
+    Parameters
+    ----------
+    xs : symbolic tensors
+        Input
+
+    Returns
+    -------
+    tensor
+
+    """
+
+    return log(add(*[exp(x) for x in xs]))
+
+
+def logsumexp(x, axis=None, keepdims=False):
+    """Compute the log of the sum of exponentials of input elements.
+
+    See ``scipy.special.logsumexp``.
+
+    Parameters
+    ----------
+    x : symbolic tensor
+        Input
+
+    axis : None or int or tuple of ints, optional
+        Axis or axes over which the sum is taken. By default axis is None,
+        and all elements are summed.
+
+    keepdims : bool, optional
+        If this is set to True, the axes which are reduced are left in the
+        result as dimensions with size one. With this option, the result will
+        broadcast correctly against the original array.
+
+    Returns
+    -------
+    tensor
+
+    """
+
+    return log(sum(exp(x), axis=axis, keepdims=keepdims))
+
+
 __all__ = [
     "max_and_argmax",
     "max",
@@ -2837,6 +2900,7 @@ __all__ = [
     "gamma",
     "gammaln",
     "psi",
+    "digamma",
     "tri_gamma",
     "chi2sf",
     "gammainc",
@@ -2852,6 +2916,9 @@ __all__ = [
     "sigmoid",
     "expit",
     "softplus",
+    "log1pexp",
+    "log1mexp",
+    "betainc",
     "real",
     "imag",
     "angle",
@@ -2885,4 +2952,6 @@ __all__ = [
     "all",
     "ptp",
     "power",
+    "logaddexp",
+    "logsumexp",
 ]
