@@ -1,26 +1,27 @@
 from collections.abc import Sequence
 from copy import copy
+from typing import List, Optional, Tuple
 
 import numpy as np
 
 import aesara
-from aesara.assert_op import Assert
 from aesara.configdefaults import config
 from aesara.graph.basic import Apply, Variable
 from aesara.graph.op import Op
 from aesara.misc.safe_asarray import _asarray
-from aesara.scalar.basic import Cast
+from aesara.scalar import ScalarVariable
 from aesara.tensor.basic import (
     as_tensor_variable,
     constant,
     get_scalar_constant_value,
     get_vector_length,
+    infer_broadcastable,
 )
-from aesara.tensor.elemwise import Elemwise
-from aesara.tensor.exceptions import NotScalarConstantError
-from aesara.tensor.random.type import RandomStateType
+from aesara.tensor.random.type import RandomType
 from aesara.tensor.random.utils import normalize_size_param, params_broadcast_shapes
+from aesara.tensor.shape import shape_tuple
 from aesara.tensor.type import TensorType, all_dtypes
+from aesara.tensor.var import TensorVariable
 
 
 def default_shape_from_params(
@@ -158,31 +159,32 @@ class RandomVariable(Op):
 
     def rng_fn(self, rng, *args, **kwargs):
         """Sample a numeric random variate."""
-        return getattr(np.random.RandomState, self.name)(rng, *args, **kwargs)
+        return getattr(rng, self.name)(*args, **kwargs)
 
     def __str__(self):
         props_str = ", ".join((f"{getattr(self, prop)}" for prop in self.__props__[1:]))
         return f"{self.name}_rv{{{props_str}}}"
 
-    def _infer_shape(self, size, dist_params, param_shapes=None):
+    def _infer_shape(
+        self,
+        size: Tuple[TensorVariable],
+        dist_params: List[TensorVariable],
+        param_shapes: Optional[List[Tuple[TensorVariable]]] = None,
+    ) -> Tuple[ScalarVariable]:
         """Compute the output shape given the size and distribution parameters.
 
         Parameters
         ----------
-        size : TensorVariable
+        size
             The size parameter specified for this `RandomVariable`.
-        dist_params : list of TensorVariable
+        dist_params
             The symbolic parameter for this `RandomVariable`'s distribution.
-        param_shapes : list of tuples of TensorVariable (optional)
+        param_shapes
             The shapes of the `dist_params` as given by `ShapeFeature`'s
             via `Op.infer_shape`'s `input_shapes` argument.  This parameter's
             values are essentially more accurate versions of ``[d.shape for d
             in dist_params]``.
 
-        Outputs
-        -------
-        shape : tuple of `ScalarVariable`
-
         """
 
         size_len = get_vector_length(size)
@@ -199,7 +201,7 @@ class RandomVariable(Op):
 
         # Broadcast the parameters
         param_shapes = params_broadcast_shapes(
-            param_shapes or [p.shape for p in dist_params], self.ndims_params
+            param_shapes or [shape_tuple(p) for p in dist_params], self.ndims_params
         )
 
         def slice_ind_dims(p, ps, n):
@@ -272,48 +274,16 @@ class RandomVariable(Op):
 
         return shape
 
-    @config.change_flags(compute_test_value="off")
-    def compute_bcast(self, dist_params, size):
-        """Compute the broadcast array for this distribution's `TensorType`.
-
-        Parameters
-        ----------
-        dist_params: list
-            Distribution parameters.
-        size: int or Sequence (optional)
-            Numpy-like size of the output (i.e. replications).
-
-        """
-        shape = self._infer_shape(size, dist_params)
-
-        # Ignore `Cast`s, since they do not affect broadcastables
-        if getattr(shape, "owner", None) and (
-            isinstance(shape.owner.op, Elemwise)
-            and isinstance(shape.owner.op.scalar_op, Cast)
-        ):
-            shape = shape.owner.inputs[0]
-
-        # Let's try to do a better job than `_infer_ndim_bcast` when
-        # dimension sizes are symbolic.
-        bcast = []
-        for s in shape:
-            s_owner = getattr(s, "owner", None)
-
-            # Get rid of the `Assert`s added by `broadcast_shape`
-            if s_owner and isinstance(s_owner.op, Assert):
-                s = s_owner.inputs[0]
-
-            try:
-                s_val = get_scalar_constant_value(s)
-            except NotScalarConstantError:
-                s_val = False
-
-            bcast += [s_val == 1]
-        return bcast
-
     def infer_shape(self, fgraph, node, input_shapes):
         _, size, _, *dist_params = node.inputs
-        _, _, _, *param_shapes = input_shapes
+        _, size_shape, _, *param_shapes = input_shapes
+
+        try:
+            size_len = get_vector_length(size)
+        except ValueError:
+            size_len = get_scalar_constant_value(size_shape[0])
+
+        size = tuple(size[n] for n in range(size_len))
 
         shape = self._infer_shape(size, dist_params, param_shapes=param_shapes)
 
@@ -330,14 +300,10 @@ class RandomVariable(Op):
     def make_node(self, rng, size, dtype, *dist_params):
         """Create a random variable node.
 
-        XXX: Unnamed/non-keyword arguments are considered distribution
-        parameters!  If you want to set `size`, `rng`, and/or `name`, use their
-        keywords.
-
         Parameters
         ----------
-        rng: RandomStateType
-            Existing Aesara `RandomState` object to be used.  Creates a
+        rng: RandomGeneratorType or RandomStateType
+            Existing Aesara `Generator` or `RandomState` object to be used.  Creates a
             new one, if `None`.
         size: int or Sequence
             Numpy-like size of the output (i.e. replications).
@@ -363,11 +329,14 @@ class RandomVariable(Op):
         )
 
         if rng is None:
-            rng = aesara.shared(np.random.RandomState())
-        elif not isinstance(rng.type, RandomStateType):
-            raise TypeError("The type of rng should be an instance of RandomStateType")
+            rng = aesara.shared(np.random.default_rng())
+        elif not isinstance(rng.type, RandomType):
+            raise TypeError(
+                "The type of rng should be an instance of either RandomGeneratorType or RandomStateType"
+            )
 
-        bcast = self.compute_bcast(dist_params, size)
+        shape = self._infer_shape(size, dist_params)
+        _, bcast = infer_broadcastable(shape)
         dtype = self.dtype or dtype
 
         if dtype == "floatX":
