@@ -10,10 +10,11 @@ from aesara.graph.op import COp
 from aesara.graph.params_type import ParamsType
 from aesara.misc.safe_asarray import _asarray
 from aesara.scalar import int32
+from aesara.tensor import _get_vector_length
 from aesara.tensor import basic as aet
 from aesara.tensor.exceptions import NotScalarConstantError
 from aesara.tensor.type import TensorType, int_dtypes, tensor
-from aesara.tensor.var import TensorConstant, TensorVariable
+from aesara.tensor.var import TensorConstant
 
 
 def register_shape_c_code(type, code, version=()):
@@ -129,6 +130,24 @@ shape = Shape()
 _shape = shape  # was used in the past, now use shape directly.
 
 
+@_get_vector_length.register(Shape)
+def _get_vector_length_Shape(op, var):
+    return var.owner.inputs[0].type.ndim
+
+
+def shape_tuple(x):
+    """Get a tuple of symbolic shape values.
+
+    This will return a `ScalarConstant` with the value ``1`` wherever
+    broadcastable is ``True``.
+    """
+    one_at = aesara.scalar.ScalarConstant(aesara.scalar.int64, 1)
+    return tuple(
+        one_at if getattr(sh, "value", sh) == 1 or bcast else sh
+        for sh, bcast in zip(shape(x), getattr(x, "broadcastable", (False,) * x.ndim))
+    )
+
+
 class Shape_i(COp):
     """
     L{Op} to return the shape of a matrix.
@@ -176,9 +195,9 @@ class Shape_i(COp):
 
     def make_node(self, x):
         if not isinstance(x, Variable):
-            raise TypeError("x must be Variable with ndim attribute", x)
+            raise TypeError(f"{x} must be Variable with ndim attribute")
         if x.ndim <= self.i:
-            raise TypeError("x has too few dimensions for Shape_i", (x, self.i))
+            raise TypeError(f"{x} has too few dimensions for Shape_i")
         return Apply(self, [x], [aesara.tensor.type.lscalar()])
 
     def perform(self, node, inp, out_, params):
@@ -479,6 +498,14 @@ class SpecifyShape(COp):
 specify_shape = SpecifyShape()
 
 
+@_get_vector_length.register(SpecifyShape)
+def _get_vector_length_SpecifyShape(op, var):
+    try:
+        return aet.get_scalar_constant_value(var.owner.inputs[1])
+    except NotScalarConstantError:
+        raise ValueError(f"Length of {var} cannot be determined")
+
+
 class Reshape(COp):
     """Perform a reshape operation of the input x to the new shape shp.
     The number of dimensions to which to reshape to (ndim) must be
@@ -513,7 +540,7 @@ class Reshape(COp):
             # It raises an error if shp is not of integer type,
             # except when shp is constant and empty
             # (in this case, shp.dtype does not matter anymore).
-            raise TypeError("Shape must be integers", shp, shp.dtype)
+            raise TypeError(f"Shape must be integers; got {shp.dtype}")
         assert shp.ndim == 1
         if isinstance(shp, TensorConstant):
             bcast = [s == 1 for s in shp.data]
@@ -543,16 +570,11 @@ class Reshape(COp):
         if len(shp) != self.ndim:
             raise ValueError(
                 (
-                    "shape argument to Reshape.perform has incorrect"
-                    f" length {len(shp)}"
-                    f", should be {self.ndim}"
-                ),
-                shp,
+                    "Shape argument to Reshape has incorrect"
+                    f" length: {len(shp)}, should be {self.ndim}"
+                )
             )
-        try:
-            out[0] = np.reshape(x, shp)
-        except Exception:
-            raise ValueError(f"Cannot reshape input of shape {x.shape} to shape {shp}")
+        out[0] = np.reshape(x, shp)
 
     def connection_pattern(self, node):
         return [[True], [False]]
@@ -565,7 +587,7 @@ class Reshape(COp):
     def R_op(self, inputs, eval_points):
         if eval_points[0] is None:
             return [None]
-        return self(eval_points[0], *inputs[1:], **dict(return_list=True))
+        return self(eval_points[0], *inputs[1:], return_list=True)
 
     def infer_shape(self, fgraph, node, ishapes):
         from aesara.tensor.math import eq, maximum, mul
@@ -643,44 +665,38 @@ class Reshape(COp):
             ]
 
     def c_code_cache_version(self):
-        return (8,)
+        return (9,)
 
     def c_code(self, node, name, inputs, outputs, sub):
-        if isinstance(node.inputs[0], TensorVariable):
-            x, shp = inputs
-            (z,) = outputs
-            sdtype = node.inputs[1].type.dtype_specs()[1]
-            fail = sub["fail"]
-            params = sub["params"]
-            return (
-                """
-            assert (PyArray_NDIM(%(shp)s) == 1);
-            npy_intp new_dims[%(params)s->ndim];
-            PyArray_Dims newshape;
-            newshape.ptr = new_dims;
-            newshape.len = %(params)s->ndim;
-            for (int ii = 0; ii < %(params)s->ndim; ++ii)
-            {
-                // -- We do not want an explicit cast here. the shp can be any
-                // -- int* dtype. The compiler will explicitly upcast it, but
-                // -- will err if this will downcast. This could happen if the
-                // -- user pass an int64 dtype, but npy_intp endup being int32.
-                new_dims[ii] = ((%(sdtype)s*)(
-                        PyArray_BYTES(%(shp)s) +
-                        ii * PyArray_STRIDES(%(shp)s)[0]))[0];
-            }
-            Py_XDECREF(%(z)s);
-            %(z)s = (PyArrayObject *) PyArray_Newshape(%(x)s, &newshape, NPY_CORDER);
-            if (!%(z)s)
-            {
-                //The error message should have been set by PyArray_Newshape
-                %(fail)s;
-            }
-            """
-                % locals()
-            )
-        else:
-            raise NotImplementedError()
+        x, shp = inputs
+        (z,) = outputs
+        fail = sub["fail"]
+        params = sub["params"]
+        return f"""
+        assert (PyArray_NDIM({shp}) == 1);
+
+        PyArray_Dims newshape;
+
+        if (!PyArray_IntpConverter((PyObject *){shp}, &newshape)) {{
+            {fail};
+        }}
+
+        if ({params}->ndim != newshape.len) {{
+            PyErr_SetString(PyExc_ValueError, "Shape argument to Reshape has incorrect length");
+            PyDimMem_FREE(newshape.ptr);
+            {fail};
+        }}
+
+        Py_XDECREF({z});
+        {z} = (PyArrayObject *) PyArray_Newshape({x}, &newshape, NPY_CORDER);
+
+        PyDimMem_FREE(newshape.ptr);
+
+        if (!{z}) {{
+            //The error message should have been set by PyArray_Newshape
+            {fail};
+        }}
+        """
 
 
 def reshape(x, newshape, ndim=None):
