@@ -2,6 +2,8 @@ import copy
 import traceback as tb
 import warnings
 from collections.abc import Iterable
+from numbers import Number
+from typing import Optional
 
 import numpy as np
 
@@ -524,59 +526,26 @@ class _tensor_py_operators:
         )
 
         # Determine if advanced indexing is needed or not.  The logic is
-        # already in `Subtensor.convert`: if it succeeds, standard indexing is
-        # used; if it fails with AdvancedIndexingError, advanced indexing is
+        # already in `index_vars_to_types`: if it succeeds, standard indexing is
+        # used; if it fails with `AdvancedIndexingError`, advanced indexing is
         # used
         advanced = False
-        axis = None
         for i, arg in enumerate(args):
             if includes_bool(arg):
                 advanced = True
-                axis = None
                 break
 
             if arg is not np.newaxis:
                 try:
-                    aet.subtensor.Subtensor.convert(arg)
+                    aet.subtensor.index_vars_to_types(arg)
                 except AdvancedIndexingError:
                     if advanced:
-                        axis = None
                         break
                     else:
                         advanced = True
-                        axis = i
 
         if advanced:
-            if (
-                axis is not None
-                and all(isinstance(a, slice) and a == slice(None) for a in args[:axis])
-                and all(
-                    isinstance(a, slice) and a == slice(None) for a in args[axis + 1 :]
-                )
-                # I.e. if the first advanced index is a tensor or NumPy array,
-                # then it can't be boolean (in order to meet this condition).
-                # How could this possibly occur; we filter for booleans above,
-                # right?
-                # and (not hasattr(args[axis], "dtype") or args[axis].dtype != "bool")
-                and isinstance(
-                    args[axis],
-                    (
-                        np.ndarray,
-                        list,
-                        TensorVariable,
-                        TensorConstant,
-                        aet.sharedvar.TensorSharedVariable,
-                    ),
-                )
-            ):
-                # If we're here, it means that an advanced index was found
-                # (e.g. an array of indices) and it was surrounded by full
-                # slices--or no slices (e.g. `x[:, :, idx, ...]`).  The
-                # `take` function/`Op` serves exactly this type of indexing,
-                # so we simply return its result.
-                return self.take(args[axis], axis)
-            else:
-                return aet.subtensor.advanced_subtensor(self, *args)
+            return aet.subtensor.advanced_subtensor(self, *args)
         else:
             if np.newaxis in args:
                 # `np.newaxis` (i.e. `None`) in NumPy indexing mean "add a new
@@ -597,6 +566,9 @@ class _tensor_py_operators:
                         pattern.append(counter)
                         counter += 1
                         new_args.append(arg)
+
+                pattern.extend(list(range(counter, self.ndim)))
+
                 view = self.dimshuffle(pattern)
                 full_slices = True
                 for arg in new_args:
@@ -617,7 +589,7 @@ class _tensor_py_operators:
             else:
                 return aet.subtensor.Subtensor(args)(
                     self,
-                    *aet.subtensor.Subtensor.collapse(
+                    *aet.subtensor.get_slice_elements(
                         args, lambda entry: isinstance(entry, Variable)
                     ),
                 )
@@ -675,6 +647,8 @@ class _tensor_py_operators:
         return aet.math.dense_dot(left, right)
 
     dot = __dot__
+    __matmul__ = __dot__
+    __rmatmul__ = __rdot__
 
     def sum(self, axis=None, dtype=None, keepdims=False, acc_dtype=None):
         """See `aesara.tensor.math.sum`."""
@@ -774,7 +748,7 @@ class _tensor_py_operators:
         return aet.math.round(self, mode)
 
     def trace(self):
-        return aet.nlinalg.trace(self)
+        return aet.linalg.trace(self)
 
     # This value is set so that Aesara arrays will trump NumPy operators.
     __array_priority__ = 1000
@@ -985,7 +959,21 @@ class TensorConstantSignature(tuple):
     no_nan = property(_get_no_nan)
 
 
-class TensorConstant(_tensor_py_operators, Constant):
+def get_unique_value(x: TensorVariable) -> Optional[Number]:
+    """Return the unique value of a tensor, if there is one"""
+    if isinstance(x, Constant):
+        data = x.data
+
+        if isinstance(data, np.ndarray) and data.ndim > 0:
+            flat_data = data.ravel()
+            if flat_data.shape[0]:
+                if (flat_data == flat_data[0]).all():
+                    return flat_data[0]
+
+    return None
+
+
+class TensorConstant(TensorVariable, Constant):
     """Subclass to add the tensor operators to the basic `Constant` class.
 
     To create a TensorConstant, use the `constant` function in this module.
@@ -994,16 +982,11 @@ class TensorConstant(_tensor_py_operators, Constant):
 
     def __init__(self, type, data, name=None):
         Constant.__init__(self, type, data, name)
-        self.tag.unique_value = None
-        if isinstance(data, np.ndarray) and data.ndim > 0:
-            flat_data = data.ravel()
-            if flat_data.shape[0]:
-                if (flat_data == flat_data[0]).all():
-                    self.tag.unique_value = flat_data[0]
 
     def __str__(self):
-        if self.tag.unique_value is not None:
-            name = f"{self.data.shape} of {self.tag.unique_value}"
+        unique_val = get_unique_value(self)
+        if unique_val is not None:
+            name = f"{self.data.shape} of {unique_val}"
         else:
             name = f"{self.data}"
         if len(name) > 20:
